@@ -56,11 +56,11 @@
 ///@{
 /// @brief Library major, minor and patch version numbers
 /// @hideinitializer
-#define OPENVDB_LIBRARY_MAJOR_VERSION_NUMBER 9
+#define OPENVDB_LIBRARY_MAJOR_VERSION_NUMBER 10
 /// @hideinitializer
 #define OPENVDB_LIBRARY_MINOR_VERSION_NUMBER 0
 /// @hideinitializer
-#define OPENVDB_LIBRARY_PATCH_VERSION_NUMBER 0
+#define OPENVDB_LIBRARY_PATCH_VERSION_NUMBER 1
 ///@}
 
 /// @note  This ifndef exists for compatibility with older versions of OpenVDB.
@@ -71,7 +71,7 @@
 #ifndef OPENVDB_ABI_VERSION_NUMBER
 /// @brief The ABI version that OpenVDB was built with
 /// @hideinitializer
-#define OPENVDB_ABI_VERSION_NUMBER 9
+#define OPENVDB_ABI_VERSION_NUMBER 10
 #endif
 
 /// @brief Library version number string of the form "<major>.<minor>.<patch>"
@@ -79,18 +79,23 @@
 /// want the compile-time version number, not the runtime version number
 /// (although the two are usually the same).
 /// @hideinitializer
-#define OPENVDB_LIBRARY_VERSION_STRING "9.0.0"
+#define OPENVDB_LIBRARY_VERSION_STRING "10.0.1"
 
 /// @brief Library version number string of the form "<major>.<minor>.<patch>abi<abi>"
 /// @details This is a macro rather than a static constant because we typically
 /// want the compile-time version number, not the runtime version number
 /// (although the two are usually the same).
 /// @hideinitializer
-#define OPENVDB_LIBRARY_ABI_VERSION_STRING "9.0.0abi9"
+#define OPENVDB_LIBRARY_ABI_VERSION_STRING "10.0.1abi10"
 
 /// @brief Library version number as a packed integer ("%02x%02x%04x", major, minor, patch)
 /// @hideinitializer
-#define OPENVDB_LIBRARY_VERSION_NUMBER 150994944
+#define OPENVDB_LIBRARY_VERSION_NUMBER 167772161
+
+/// @brief Where this version was compiled from if it comes from a
+/// git repo.
+#define OPENVDB_PACKAGE_URL ""
+#define OPENVDB_PACKAGE_REVISION ""
 
 /// @brief The version namespace name for this library version
 /// @hideinitializer
@@ -111,9 +116,9 @@
 ///
 /// where X, Y and N are the major, minor and ABI version numbers, respectively.
 #if OPENVDB_ABI_VERSION_NUMBER == OPENVDB_LIBRARY_MAJOR_VERSION_NUMBER
-    #define OPENVDB_VERSION_NAME v9_0
+    #define OPENVDB_VERSION_NAME v10_0
 #else
-    #define OPENVDB_VERSION_NAME v9_0abi9
+    #define OPENVDB_VERSION_NAME v10_0abi10
 #endif
 
 /* Denotes whether VDB was built with IMath Half support */
@@ -133,6 +138,11 @@
 #define OPENVDB_USE_ZLIB
 #endif
 
+/* Denotes whether VDB was built with Delayed Loading support */
+#ifndef OPENVDB_USE_DELAYED_LOADING
+#define OPENVDB_USE_DELAYED_LOADING
+#endif
+
 /* Denotes whether VDB was built with explicit template instantiation */
 #ifndef OPENVDB_USE_EXPLICIT_INSTANTIATION
 #define OPENVDB_USE_EXPLICIT_INSTANTIATION
@@ -186,10 +196,10 @@
     // This can be suppressed by defining OPENVDB_USE_FUTURE_ABI_<VERSION>=ON.
     // Note that, whilst the VDB CMake does not allow this option to be hit,
     // it exists to propagate this message to downstream targets
-    #if OPENVDB_ABI_VERSION_NUMBER == 10
-        #ifndef OPENVDB_USE_FUTURE_ABI_10
-            PRAGMA(message("NOTE: ABI = 10 is still in active development and has not been finalized, "
-                "define OPENVDB_USE_FUTURE_ABI_10 to suppress this message"))
+    #if OPENVDB_ABI_VERSION_NUMBER == 11
+        #ifndef OPENVDB_USE_FUTURE_ABI_11
+            PRAGMA(message("NOTE: ABI = 11 is still in active development and has not been finalized, "
+                "define OPENVDB_USE_FUTURE_ABI_11 to suppress this message"))
         #endif
     #else
         #error expected OPENVDB_ABI_VERSION_NUMBER <= OPENVDB_LIBRARY_MAJOR_VERSION_NUMBER
@@ -200,15 +210,15 @@
 // directive. This can be suppressed by defining OPENVDB_USE_DEPRECATED_ABI_<VERSION>.
 // Note that, whilst the VDB CMake does not allow this option to be hit,
 // it exists to propagate this message to downstream targets
-#ifndef OPENVDB_USE_DEPRECATED_ABI_6
-    #if OPENVDB_ABI_VERSION_NUMBER == 6
-        PRAGMA(message("NOTE: ABI = 6 is deprecated, define OPENVDB_USE_DEPRECATED_ABI_6 "
+#ifndef OPENVDB_USE_DEPRECATED_ABI_8
+    #if OPENVDB_ABI_VERSION_NUMBER == 8
+        PRAGMA(message("NOTE: ABI = 8 is deprecated, define OPENVDB_USE_DEPRECATED_ABI_7 "
             "to suppress this message"))
     #endif
 #endif
-#ifndef OPENVDB_USE_DEPRECATED_ABI_7
-    #if OPENVDB_ABI_VERSION_NUMBER == 7
-        PRAGMA(message("NOTE: ABI = 7 is deprecated, define OPENVDB_USE_DEPRECATED_ABI_7 "
+#ifndef OPENVDB_USE_DEPRECATED_ABI_9
+    #if OPENVDB_ABI_VERSION_NUMBER == 9
+        PRAGMA(message("NOTE: ABI = 9 is deprecated, define OPENVDB_USE_DEPRECATED_ABI_8 "
             "to suppress this message"))
     #endif
 #endif
@@ -283,6 +293,8 @@ inline constexpr const char* getLibraryVersionString() { return OPENVDB_LIBRARY_
 inline constexpr const char* getLibraryAbiVersionString() {
     return OPENVDB_LIBRARY_ABI_VERSION_STRING;
 }
+inline constexpr const char* getPackageUrl() { return OPENVDB_PACKAGE_URL; }
+inline constexpr const char* getPackageRevision() { return OPENVDB_PACKAGE_REVISION; }
 
 
 struct VersionId {
